def get_tables(self):
    table_names = [
        "credentials",
        "sourcecategories",
        "sourcesubcategories",
        "sourcedatamodel",
        "userconfig",
        "configurationentry",
        "linkdatamodel",
        "domains",
        "usertags",
        "compactedtags",
        "usercompactedtags",
        "entrycompactedtags",
        "uservotes",
        "browser",
        "entryrules",
        "dataexport",
        "gateway",
        "modelfiles",
        "readlater",
        "searchview",
        "socialdata",
        "blockentry",
        "blockentrylist",
        "usercomments",
        "userbookmarks",
        "usersearchhistory",
        "userentrytransitionhistory",
        "userentryvisithistory",
    ]
    return tables


def get_backup_tables(self):
    table_names = [
        "credentials",
        "sourcecategories",
        "sourcesubcategories",
        "sourcedatamodel",
        "userconfig",
        "configurationentry",
        "linkdatamodel",
        "domains",
        "usertags",
        "compactedtags",
        "usercompactedtags",
        "entrycompactedtags",
        "uservotes",
        "browser",
        "entryrules",
        "dataexport",
        "gateway",
        "modelfiles",
        "readlater",
        "searchview",
        "socialdata",
        # "blockentry", # do not backup it, the list has to be reinitialized each time
        "blockentrylist",
        "usercomments",
        "userbookmarks",
        "usersearchhistory",
        "userentrytransitionhistory",
        "userentryvisithistory",
    ]
    return tables
